(:
 : Copyright (c) 2014-2015 Jean-Marc Mercier
  : Licensed under the Apache License, Version 2.0 (the "License");
 : you may not use this file except in compliance with the License.
 : You may obtain a copy of the License at
 :
 :     http://www.apache.org/licenses/LICENSE-2.0
 :
 : Unless required by applicable law or agreed to in writing, software
 : distributed under the License is distributed on an "AS IS" BASIS,
 : WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 : See the License for the specific language governing permissions and
 : limitations under the License.
 :)
module namespace idmap = "http://www.crimere.com/idmap";
import module namespace set="http://www.crimere.com/set" at "set.xqm";
declare default function namespace "http://www.crimere.com/idmap";

declare function  idmap()                                                    {fn:error(xs:QName('idmap'),'without id function, please use a classical map instead to avoid useless overhead')};
declare function  idmap($id as function(item()) as xs:anyAtomicType)         {set:set($id)};
declare function  idmap($idmap, $pairs)                                         
                                                                                {let $id := set:getid($idmap) let $map := set:getmap($idmap) 
                                                                                return set:pair(map:new( ($map, for $pair in $pairs return map:entry($id(set:first($pair)),$pair) ) ), $id)};
declare function  empty         ($idmap as function() as function(*))           {set:empty($idmap)};
declare function  count         ($idmap as function() as function(*))           {set:count($idmap)};
declare function  get           ($idmap as function() as function(*))           {set:get($idmap)};
declare function  get_keys      ($idmap as function() as function(*))           {for $el in get($idmap) return set:first($el)};
declare function  get_values    ($idmap as function() as function(*))           {for $el in get($idmap) return set:second($el)};
declare function  get           ($idmap as function() as function(*),$keys)     {let $map := set:getmap($idmap) return for $id in getid($idmap,$keys) return map:get($map,$id)};
declare function  get_value     ($idmap as function() as function(*),$keys)     {for $pair in get($idmap,$keys) return set:second($pair)};

declare function  contains      ($idmap as function() as function(*),$key as item()) as xs:boolean                  {set:contains($idmap,$key)}; 
declare function  remove        ($idmap as function() as function(*),$keys)                                         {set:remove($idmap,$keys)}; 

declare function  getid         ($idmap as function() as function(*)) as function(item()) as xs:anyAtomicType *     {set:getid($idmap)};
declare function  getid         ($idmap as function() as function(*),$els) as xs:anyAtomicType *                    {set:getid($idmap,$els)};


(:-----------------profiler functions---------------:) 
declare function integer_sequence($nb)                  {for $i in 1 to xs:integer($nb) return $i};
declare function integer_pair_sequence($nb)             {for $i in 1 to xs:integer($nb) return set:pair($i,$i)};
declare function integer_entry_sequence($nb)            {for $i in 1 to xs:integer($nb) return map:entry($i,$i)};
declare function node_sequence($nb)                     {for $i in 1 to xs:integer($nb) return element{"dummy"}{$i}};
declare function node_pair_sequence($nb)                {for $i in 1 to xs:integer($nb) return set:pair(element{"dummy"}{$i},element{"dummy"}{$i})};

declare function profile($i,$j)
{
    for $nb in $i to $j
    let $pow := math:pow(2,$nb)
    let $dummy :=
    (
    fn:trace("","<test>")
    ,fn:trace("","<nb>"),fn:trace(math:pow(2,$nb),""),fn:trace("","</nb>")
    ,fn:trace("","<integer_set>"),fn:trace(let $dummy := prof:time(count(
            set:set(set:set(function($el) as xs:anyAtomicType {$el}),integer_sequence($pow))
    )) return (),""),fn:trace("","</integer_set>")
    ,fn:trace("","<node_set>"),fn:trace(let $dummy := prof:time(count(
        set:set(set:set(function($el) as xs:anyAtomicType {xs:integer(fn:data($el))}), node_sequence($pow))
     )) return (),""),fn:trace("","</node_set>")
    ,fn:trace("","<integer_map>"),fn:trace(let $dummy := prof:time(idmap:count(
        idmap:idmap(idmap:idmap(function($el) as xs:anyAtomicType {$el}),integer_pair_sequence($pow))
     )) return (),""),fn:trace("","</integer_map>")
    ,fn:trace("","<Basex_integer_map>"),fn:trace(let $dummy := prof:time(map:size(
        map:new( (map:new(), integer_entry_sequence($pow)) )
     )) return (),""),fn:trace("","</Basex_integer_map>")
    ,fn:trace("","<node_map>"),fn:trace(let $dummy := prof:time(idmap:count(
        idmap:idmap(idmap:idmap(function($el) as xs:anyAtomicType {xs:integer($el)}),node_pair_sequence($pow))
     )) return (),""),fn:trace("","</node_map>")
    ,fn:trace("","</test>") 
    )
    return ()
};                                                                                                